/*
 * Process.h
 *
 *  Created on: 23.05.2009
 *      Author: breeze
 */

#ifndef PROCESS_H_
#define PROCESS_H_

#include <flysystemmonitor_export.h>

#include <vector>
#include <tr1/memory>
#include <QString>
#include <QVariant>

namespace gtop_wrappers
{

/// Интерфейс для параметров процесса
struct FLY_SYSTEM_MONITOR_EXPORT IProcessParam
{
    /// Возвращает значение для отображения в списке процессов
    virtual QVariant    getDisplayValue() const = 0;
    /// Возвращает значение используемое для сортировки
    virtual QVariant    getSortValue() const = 0;
    virtual ~IProcessParam() {}
};
typedef std::tr1::shared_ptr<IProcessParam> PtrProcessParam;
typedef std::vector<PtrProcessParam>        PtrProcessParams;

//class ProcessFiller;
struct FLY_SYSTEM_MONITOR_EXPORT Process
{
    /// Чтобы кто-то менял размер m_Params
    friend class ProcessFiller;

    enum WellKnownColumns
    {
        /// Название запущенного процесса
        COLUMN_NAME = 0,
        /// Загрузка процессора в тиках
        COLUMN_CPU,
        /// Общая память процесса
        COLUMN_MEM_TOTAL,
        /// Резидентная память процесса
        COLUMN_MEM_RESIDENT,
        /// PID
        COLUMN_ID,
        /// \brief Состояние процесса
        ///
        /// RUNNING     1
        /// INTERRUPTIBLE   2
        /// UNINTERRUPTIBLE 4
        /// ZOMBIE      8
        /// STOPPED     16
        /// SWAPPING        32
        /// DEAD        64
        COLUMN_STATE,
        COLUMN_PRIORITY,
//      COLUMN_NICE,
        COLUMN_UID,
        COLUMN_GID,
        /// PID родительского процесса
        COLUMN_PPID
    };

    static QString getStrState(int _state);
    Process( int _pid = -1 );
    int pid() const;

    size_t  size() const;
    const PtrProcessParam   getParam(size_t _index) const;
    void setParam( size_t _index, const PtrProcessParam& );

    int getCpuLoad() const;
    int getPPid() const;
    /// Имя запущенной программы
    QString getName() const;
    int getNice() const;

protected:
    int         m_pid;
    PtrProcessParams    m_Params;
};
typedef std::map<pid_t, Process>    ProcessMap;

enum FLY_SYSTEM_MONITOR_EXPORT ProcessTypes
{
    ALL = 0,
    OWN,
    ACTIVE
};

struct FLY_SYSTEM_MONITOR_EXPORT NullProcessParam : public IProcessParam
{
    NullProcessParam() {}
    void        setValue( const QVariant& ) {}

    QVariant    getDisplayValue() const {
        return QVariant();
    }
    QVariant    getSortValue() const {
        return QVariant();
    }
};

struct FLY_SYSTEM_MONITOR_EXPORT SimpleProcessParam : public IProcessParam
{
    SimpleProcessParam( const QVariant& _value ) : m_Value(_value) {}
    void        setValue( const QVariant& _value ) {
        m_Value = _value;
    }

    QVariant    getDisplayValue() const {
        return m_Value;
    }
    QVariant    getSortValue() const {
        return m_Value;
    }
protected:
    QVariant    m_Value;
};

struct FLY_SYSTEM_MONITOR_EXPORT IntProcessParam : public SimpleProcessParam
{
    IntProcessParam( int _sortValue )
        : SimpleProcessParam(_sortValue), m_SortValue(_sortValue) { }
    IntProcessParam( const QVariant& _value, int _sortValue )
        : SimpleProcessParam(_value), m_SortValue(_sortValue) { }
    void        setValues( const QVariant& _value, int _sortValue ) {
        setValue(_value);
        m_SortValue = _sortValue;
    }

    QVariant    getSortValue() const {
        return m_SortValue;
    }
    int     getRealSortValue() const {
        return m_SortValue;
    }
protected:
    int m_SortValue;
};

struct FLY_SYSTEM_MONITOR_EXPORT CpuProcessParam : public SimpleProcessParam
{
    CpuProcessParam( int _cpuTicks )
        : SimpleProcessParam("---"), m_CpuTicks(_cpuTicks), m_SortValue(0) {}
    void        setSortValue( double _sortValue )
    {
        setValue(QString::number(_sortValue, 'f', 1) + " %");
        m_SortValue = _sortValue;
    }

    QVariant    getSortValue() const {
        return (int)(m_SortValue*1000.0);
    }
    double      getCpuTicks() const {
        return m_CpuTicks;
    }
protected:
    int m_CpuTicks;
    double  m_SortValue;
};

}

#endif /* PROCESS_H_ */
